### Name: ucminf
### Title: General-purpose unconstrained non-linear optimization
### Aliases: ucminf
### Keywords: optimize nonlinear

### ** Examples

## Rosenbrock Banana function
fR = function(x) { (1-x[1])^2+100*(x[2]-x[1]^2)^2 }
gR = function(x) { c( -400*x[1]*(x[2]-x[1]*x[1]) - 2*(1-x[1]),
                       200*(x[2]-x[1]*x[1])) }
#  Find minimum
ucminf(par = c(2,.5), fn = fR, gr=gR)

#  Compare hessian approximations
ucminf(par = c(2,.5), fn = fR, gr=gR, hessian=1)$hessian
ucminf(par = c(2,.5), fn = fR, gr=gR, hessian=3)$hessian

#  Compare run times with optim's BFGS method
#  (chosen convergence criteria result in similar accuracy)
system.time( for(i in 1:500)
               ucminf(par = c(2,0.5), fn = fR, gr=gR)
           )
system.time( for(i in 1:500)
               optim(par = c(2,0.5), fn = fR, gr=gR,method='BFGS')
           )

## Quadratic function
fQ = function(x) { sum((4*x-1)^2) }
gQ = function(x) { 32*x -8}

#  Find minimum with too small stepmax and print trace
ucminf(par = c(20.5,20.0), fn = fQ, gr = gQ, 
       control=list(stepmax=1,trace=TRUE))

#  The same again with a larger stepmax
ucminf(par = c(20.5,20.0), fn = fQ, gr = gQ,
       control=list(stepmax=100,trace=TRUE))



